/* $Id: ContactForm.css,v 1.1.1.1 2003/02/03 08:22:38 Gunnar Hjalmarsson Exp $ */

p, h3, body { font-family: arial, helvetica, sans-serif }
p, body { background: #f0f0f0; color: black }
body { margin-left: 15% }
h3 { background: #f0f0f0; color: #4c5ea8 }
input, textarea {
    font-family: 'courier new', monospace;
    font-size: 10pt;
    background: white;
    color: black
}
p { font-size: 10pt; margin-top: 0; margin-bottom: 0.1em }
p.error { font-weight: bold; color: #cc6600; background: none }
p.returnlink { margin-top: 1em }
td p.returnlink { text-align: center }
.center { text-align: center }
td { text-align: left }
input.button {
    font-family: arial, helvetica, sans-serif;
    font-size: 12pt;
    font-weight: bold;
    padding-top: 0.1em;
    background: #ddeeff;
    color: black
}
input.button:hover { background: white; color: black }
a         { color: #4c5ea8; background: none }
a:visited { color: #4c5ea8; background: none }
a:active  { color: #cc6600; background: none }
a:hover   { color: #cc6600; background: none }
